"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ui_const_1 = require("./ui_const");
let changeUrlCallback = {};
var params = {};
window.addEventListener("popstate", (e) => {
    for (let key in changeUrlCallback) {
        changeUrlCallback[key](key, e, "popstate");
    }
});
window.addEventListener("changeurlstate", (e) => {
    params = {};
    for (let key in changeUrlCallback) {
        changeUrlCallback[key](key, e.detail, "changeurlstate");
    }
});
function onChangeUrlState(key, cb, force = false) {
    if (!changeUrlCallback[key] || force) {
        changeUrlCallback[key] = cb;
    }
}
class EleNode {
    constructor() {
        this.attributes = {};
        this.childs = [];
        this.status = true;
        this.elseEle = document.createTextNode("");
        this.forListEle = [];
    }
}
var ConstVar = {
    __for: "__for",
    __if: "__if",
    __routerpath: "__routerpath",
    __routerlink: "__routerlink",
    __object: "__o-",
    __event: "__e-",
    textNode: "#text",
    commentNode: "#comment",
    onChangeAttribute: "onChangeAttribute",
    containerNode: "container"
};
class UILite extends HTMLElement {
    constructor() {
        super();
        this.routerLink = {};
        this.vartualElements = null;
        this.vartualNode = new EleNode();
        this.params = {};
        let id = Math.random();
        onChangeUrlState(id, (key, data, ename) => {
            // this.rerender();
            let s = this.renderRouterPath();
            if (this.routerPathChange) {
                this.routerPathChange(key, data, ename, s);
            }
        });
    }
    setpath(state, title, path) {
        window.history.pushState(state, title, path);
        var event = new CustomEvent("changeurlstate", {
            detail: { state, title, path }
        });
        window.dispatchEvent(event);
    }
    renderRouterPath() {
        let matchStatus = false;
        for (let path in this.routerLink) {
            let node = this.routerLink[path];
            let val = this.comparePath(path);
            if (val.status) {
                matchStatus = true;
            }
            if (val.status != node.status) {
                if (val.status) {
                    node.elseEle.parentNode.insertBefore(node.ele, node.elseEle);
                    node.elseEle.parentNode.removeChild(node.elseEle);
                    node["status"] = true;
                    this.updateRender(null, node);
                }
                else {
                    node.ele.parentNode.insertBefore(node.elseEle, node.ele);
                    node.ele.parentNode.removeChild(node.ele);
                    node["status"] = false;
                }
            }
        }
        return matchStatus;
    }
    setCookie(cname, cvalue, exdays) {
        var d = new Date();
        d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
        var expires = "expires=" + d.toUTCString();
        document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/";
    }
    getCookie(cname) {
        var name = cname + "=";
        var decodedCookie = decodeURIComponent(document.cookie);
        var ca = decodedCookie.split(';');
        for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') {
                c = c.substring(1);
            }
            if (c.indexOf(name) == 0) {
                return c.substring(name.length, c.length);
            }
        }
        return "";
    }
    getQueryParams(query) {
        var qs = window.location.href;
        var index = qs.indexOf("?");
        qs = qs.substring(index, qs.length);
        qs = qs.split('+').join(' ');
        var params = {}, tokens, re = /[?&]?([^=]+)=([^&]*)/g;
        while (tokens = re.exec(qs)) {
            params[decodeURIComponent(tokens[1])] = decodeURIComponent(tokens[2]);
        }
        return params[query];
    }
    cloneEleNode(rootnode, node) {
        var _a;
        node.ele = rootnode.ele.cloneNode();
        node.elseEle = document.createTextNode("");
        node.forListEle = [];
        if (((_a = rootnode.childs) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            rootnode.childs.forEach((ch, i) => {
                this.cloneEleNode(ch, node.childs[i]);
            });
        }
    }
    manuculateNode(rootNode, node) {
        if (rootNode.nodeName.toLowerCase() == ConstVar.containerNode) {
            rootNode.innerHTML = this.innerHtml;
        }
        if (rootNode) {
            node.parent = rootNode.parent;
            if (rootNode.nodeName != ConstVar.textNode) {
                node.ele = document.createElement(rootNode.nodeName);
                let attributes = rootNode.getAttributeNames();
                attributes.forEach((d) => {
                    node.attributes[d] = rootNode.getAttribute(d);
                });
                if (rootNode.childNodes.length > 0) {
                    rootNode.childNodes.forEach((ele) => {
                        if (ele.nodeName != ConstVar.commentNode) {
                            let linkNode = new EleNode();
                            node.childs.push(linkNode);
                            this.manuculateNode(ele, linkNode);
                        }
                    });
                }
            }
            else {
                node.ele = document.createTextNode(rootNode.wholeText);
                node.text = rootNode.wholeText;
            }
        }
    }
    replaceText(str, __fs = {}) {
        return str.replace(/{[\w\d\s+-/*%|&'=!"`()\[\]]+}/g, (match, number) => {
            let expression = match.substring(1, match.length - 1);
            let evals = null;
            let str = `evals=${expression}`;
            eval(str);
            return evals;
        });
    }
    comparePath(path) {
        let urlPathA = window.location.pathname.split("/");
        if (path.indexOf("/") != 0) {
            path = "/" + path;
        }
        let pathA = path.split("/");
        let params = {};
        let status = true;
        for (let i = 0; i < pathA.length; i++) {
            if (pathA[i].indexOf(":") == 0) {
                let key = pathA[i].substring(1);
                params[key] = urlPathA[i];
                continue;
            }
            if (pathA[i] != urlPathA[i]) {
                status = false;
                params = {};
                break;
            }
        }
        return {
            status,
            params
        };
    }
    initRender(node) {
        //let eleStack=this.vartualNode.childs
        //eleStack.push(this.vartualNode);
    }
    renderPage(parentNode, node, __fs = {}) {
        if (node.ele.nodeName === ConstVar.textNode) {
            node.ele.textContent = this.replaceText(node.text, __fs);
        }
        for (let key in node.attributes) {
            if (key.search(ConstVar.__if) == 0) {
                let ifExpr = this.replaceText(node.attributes[key]);
                let val = eval(ifExpr);
                if (!val) {
                    parentNode.appendChild(node.elseEle);
                    return false;
                }
            }
            else if (key.search(ConstVar.__routerpath) == 0) {
                this.routerLink[node.attributes[key]] = node;
                // let val= this.comparePath(node.attributes[key]);
                //  if(!val.status){
                //    parentNode.appendChild(node.elseEle)
                //    return false; 
                //  }
                // if(!params){
                //   params={}
                // }
                // params={...params,...val.params};
            }
            else if (key.search(ConstVar.__for) == 0) {
                let expres = node.attributes[key].split(";");
                let exp1 = expres[0].split(" in ").map((d) => d.trim());
                let exp2 = expres[1].split("=").map((d) => d.trim());
                let arrExp = this.replaceText(exp1[1]);
                let arrVar = eval(arrExp);
                //parentNode.removeChild(parentNode.lastChild);
                let forCount = 0;
                node.forListEle = [];
                node["data"] = arrVar;
                arrVar.forEach((for_da, for_index) => {
                    let forNewNode = JSON.parse(JSON.stringify(node));
                    // forNewNode.ele=node.ele.cloneNode(true);
                    delete forNewNode.attributes.__for;
                    __fs[exp1[0]] = for_da;
                    if (exp2[0])
                        __fs[exp2[0]] = for_index;
                    this.cloneEleNode(node, forNewNode);
                    if (this.renderPage(parentNode, forNewNode, __fs)) {
                        forNewNode["index"] = forCount;
                        forCount++;
                        forNewNode["status"] = true;
                        node.forListEle.push(forNewNode);
                    }
                    else {
                        forNewNode["index"] = -1;
                        forNewNode["status"] = false;
                    }
                });
                let nnode = new EleNode();
                nnode.ele = document.createTextNode("");
                node.forListEle.push(nnode);
                parentNode.appendChild(nnode.ele);
                node.forLength = forCount;
                return true;
            }
            else if (key.search(ConstVar.__routerlink) == 0) {
                let val = node.attributes[key];
                node.ele["__fs"] = Object.assign({}, __fs);
                if (!node.ele["props"]) {
                    node.ele["props"] = {};
                }
                if (!node.ele["props"]["click"]) {
                    //let fsObj={...__fs};
                    let fun = (__event) => {
                        // eval(val);
                        let __fs = {};
                        if (__event.target["__fs"])
                            __fs = __event.target["__fs"];
                        let path = eval(val);
                        this.setpath({ path }, path, path);
                    };
                    node.ele.addEventListener("click", (e) => {
                        fun(e);
                    });
                    node.ele["props"]["click"] = true;
                }
            }
            else if (key.search(ConstVar.__object) == 0) {
                let newAtr = key.substring(4);
                let objExp = this.replaceText(node.attributes[key], __fs);
                let val = eval(objExp);
                if (!node.ele["props"]) {
                    node.ele["props"] = {};
                }
                node.ele["props"][newAtr] = val;
                node.ele.setAttribute(newAtr, val);
                // console.log(node.ele);
            }
            else if (key.search(ConstVar.__event) == 0) {
                let newAtr = key.substring(4);
                let val = node.attributes[key];
                node.ele["__fs"] = Object.assign({}, __fs);
                if (!node.ele["props"]) {
                    node.ele["props"] = {};
                }
                if (!node.ele["props"][newAtr]) {
                    //let fsObj={...__fs};
                    let fun = (__event) => {
                        let __fs = {};
                        if (__event.target["__fs"])
                            __fs = __event.target["__fs"];
                        eval(val);
                    };
                    node.ele.addEventListener(newAtr, (e) => {
                        fun(e);
                    });
                    node.ele.props[newAtr] = (data) => {
                        var event = new CustomEvent(newAtr, {
                            detail: data
                        });
                        node.ele.dispatchEvent(event);
                    };
                }
            }
            else {
                node.ele.setAttribute(key, node.attributes[key]);
            }
        }
        parentNode.appendChild(node.ele);
        let nodeCount = 0;
        node.childs.forEach((ch) => {
            if (this.renderPage(parentNode.lastChild, ch, __fs)) {
                ch["index"] = nodeCount;
                nodeCount++;
                ch["status"] = true;
            }
            else {
                ch["index"] = -1;
                ch["status"] = false;
            }
        });
        return true;
    }
    updateRender(nodePtr, node, __fs = {}) {
        if (node.ele.nodeName === ConstVar.textNode) {
            let strText = this.replaceText(node.text, __fs);
            if (node.ele.textContent != strText)
                node.ele.textContent = strText;
            return true;
        }
        if (!node.ele["props"]) {
            node.ele["props"] = {};
        }
        if (node.attributes[ConstVar.__for]) {
            let expres = node.attributes[ConstVar.__for].split(";");
            let exp1 = expres[0].split(" in ").map((d) => d.trim());
            let exp2 = expres[1].split("=").map((d) => d.trim());
            let arrExp = this.replaceText(exp1[1]);
            let arrVar = eval(arrExp);
            let forCount = 0;
            if (node.data != arrVar) {
                let textNode = node.forListEle.pop();
                //let tempEle=node.forListEle;
                // node.forListEle=node.forListEle.filter((n)=>n.ele.nodeName!=ConstVar.textNode);
                arrVar.forEach((for_da, for_index) => {
                    __fs[exp1[0]] = for_da;
                    if (exp2[0])
                        __fs[exp2[0]] = for_index;
                    if (node.forListEle[for_index]) {
                        this.updateRender(null, node.forListEle[for_index], __fs);
                        forCount++;
                    }
                    else {
                        let forNewNode = JSON.parse(JSON.stringify(node));
                        delete forNewNode.attributes.__for;
                        this.cloneEleNode(node, forNewNode);
                        if (this.updateRender(forNewNode.ele, forNewNode, __fs)) {
                            textNode.ele.parentNode.insertBefore(forNewNode.ele, textNode.ele);
                            forNewNode["index"] = forCount;
                            forCount++;
                            forNewNode["status"] = true;
                            node.forListEle.push(forNewNode);
                        }
                        else {
                            forNewNode["index"] = -1;
                            forNewNode["status"] = false;
                        }
                    }
                });
                node.forListEle.forEach((n, i) => {
                    if (forCount <= i)
                        n.ele.parentNode.removeChild(n.ele);
                });
                node.forListEle.push(textNode);
                node["data"] = arrVar;
            }
            node.forLength = forCount;
            return true;
        }
        if (node.attributes[ConstVar.__if]) {
            let ifExpr = this.replaceText(node.attributes[ConstVar.__if]);
            let val = eval(ifExpr);
            if (val != node.status) {
                if (val) {
                    node.elseEle.parentNode.insertBefore(node.ele, node.elseEle);
                    node.elseEle.parentNode.removeChild(node.elseEle);
                }
                else {
                    node.ele.parentNode.insertBefore(node.elseEle, node.ele);
                    node.ele.parentNode.removeChild(node.ele);
                    return false;
                }
            }
            else if (!val) {
                return false;
            }
        }
        if (node.attributes[ConstVar.__routerpath]) {
            this.routerLink[node.attributes[ConstVar.__routerpath]] = node;
            if (!node.status) {
                return false;
            }
        }
        if (node.attributes[ConstVar.__routerlink]) {
            let val = node.attributes[ConstVar.__routerlink];
            node.ele["__fs"] = Object.assign({}, __fs);
            if (!node.ele["props"]) {
                node.ele["props"] = {};
            }
            if (!node.ele["props"]["click"]) {
                //let fsObj={...__fs};
                let fun = (__event) => {
                    // eval(val);
                    let __fs = {};
                    if (__event.target["__fs"])
                        __fs = __event.target["__fs"];
                    let path = eval(val);
                    this.setpath({ path }, path, path);
                };
                node.ele.addEventListener("click", (e) => {
                    fun(e);
                });
                node.ele["props"]["click"] = true;
            }
        }
        let attributeChangeObject = {};
        let attributeChange = false;
        for (let key in node.attributes) {
            if (key.search(ConstVar.__object) == 0) {
                let newAtr = key.substring(4);
                let objExp = this.replaceText(node.attributes[key], __fs);
                let val = eval(objExp);
                let oldVal = node.ele["props"][newAtr];
                if (oldVal != val) {
                    if (node.ele[ConstVar.onChangeAttribute]) {
                        attributeChangeObject[newAtr] = { oldValue: oldVal, newValue: val };
                        attributeChange = true;
                    }
                    node.ele["props"][newAtr] = val;
                    node.ele.setAttribute(newAtr, val);
                    // console.log(node.ele);
                }
            }
            else if (key.search(ConstVar.__event) == 0) {
                let newAtr = key.substring(4);
                let val = node.attributes[key];
                if (!node.ele[ConstVar.__event]) {
                    node.ele[ConstVar.__event] = {};
                }
                if (!node.ele["props"]) {
                    node.ele["props"] = {};
                }
                //let fsObj={...__fs};
                node.ele["__fs"] = Object.assign({}, __fs);
                if (!node.ele["props"][newAtr]) {
                    let fun = (__event) => {
                        let __fs = {};
                        if (__event.target["__fs"])
                            __fs = __event.target["__fs"];
                        eval(val);
                    };
                    node.ele.addEventListener(newAtr, (e) => {
                        fun(e);
                    });
                    node.ele.props[newAtr] = (data) => {
                        var event = new CustomEvent(newAtr, {
                            detail: data
                        });
                        node.ele.dispatchEvent(event);
                    };
                }
            }
        }
        if (attributeChange && node.ele[ConstVar.onChangeAttribute]) {
            node.ele[ConstVar.onChangeAttribute](attributeChangeObject);
        }
        let nodeCount = 0;
        node.childs.forEach((ch) => {
            if (nodePtr) {
                this.renderPage(nodePtr, ch, __fs);
            }
            else if (this.updateRender(null, ch, __fs)) {
                ch["index"] = nodeCount;
                nodeCount++;
                ch["status"] = true;
            }
            else {
                ch["index"] = -1;
                ch["status"] = false;
            }
        });
        return true;
    }
    updateComponent() {
        this.updateRender(null, this.vartualNode);
    }
    rerender() {
        this.innerHTML = "";
        let count = 0;
        this.vartualNode.childs.forEach((d, i) => {
            if (this.renderPage(this, d)) {
                d["index"] = count;
                count++;
                d["status"] = true;
            }
            else {
                d["index"] = -1;
                d["status"] = false;
            }
        });
        //console.log(this.vartualNode);
    }
    connectedCallback() {
        if (!this.vartualElements) {
            // console.log("new elements",this)
            if (this.afterInit) {
                this.afterInit();
            }
            this.innerHtml = this.innerHTML;
            this.vartualElements = document.createElement(null);
            this.vartualElements.innerHTML = this[ui_const_1.TEMPLATE];
            this.manuculateNode(this.vartualElements, this.vartualNode);
            // console.log(this.vartualNode);
            if (this.init) {
                this.init();
            }
            this.rerender();
            this.renderRouterPath();
        }
        else {
            // console.log("elements change",this)
            if (this.reInit) {
                this.reInit();
            }
        }
    }
    adoptedCallback() {
        //console.log("move to new page=>",this)
    }
    elementInit() {
        //console.log(this)
        // this.rerender();
    }
}
exports.default = UILite;
//# sourceMappingURL=UILite.js.map